#ifndef EMUSIC_H
#define EMUSIC_H

#ifdef EAPI
#undef EAPI
#endif
#ifdef WIN32
# ifdef BUILDING_DLL
#  define EAPI __declspec(dllexport)
# else
#  define EAPI __declspec(dllimport)
# endif
#else
# ifdef __GNUC__
#  if __GNUC__ >= 4
#   define EAPI __attribute__ ((visibility("default")))
#  else
#   define EAPI
#  endif
# else
#  define EAPI
# endif
#endif

#include <Ecore.h>
#include <Ecore_Data.h>
#include <Ecore_File.h>
#include <dlfcn.h>


typedef struct _Emusic_Metadata        Emusic_Metadata;
typedef struct _Emuais_Capabilities    Emusic_Capabilities;
typedef struct _Emusic_Playlist_Item   Emusic_Playlist_Item;
typedef struct _Emusic_Playlist        Emusic_Playlist;
typedef struct _Emusic_Controller      Emusic_Controller;
typedef enum   _Emusic_Controller_Mode Emusic_Controller_Mode;

enum _Emusic_Controller_Mode
{
  EMUSIC_CONTROLLER_MODE_NULL,
  EMUSIC_CONTROLLER_MODE_PLAYER,
  EMUSIC_CONTROLLER_MODE_TRANSCODER,
  EMUSIC_CONTROLLER_MODE_STREAMER
};

struct _Emusic_Metadata
  {
    char  *title;
    char  *artist;
    char  *album;
    int   track_number;
    int   number_of_track_on_album;
    char  *comment;
    int   year;
    char  *genre;
    char  *uri;
  };

struct _Emusic_Capabilities
  {
    unsigned char can_play : 1;
    unsigned char can_stream : 1;
    unsigned char can_trancode : 1;
  };


struct _Emusic_Playlist_Item
  {
    int   id;                /*< song id */
    char *name;              /*< song name */
    //	Emusic_Metadata *meta;   /*< metadatas */
  };

struct _Emusic_Playlist
  {
    char *name;
    Ecore_List *files;
    int index;
  };

struct _Emusic_Controller
  {
    char *name;
    Emusic_Controller_Mode mode;
    Ecore_List *playlists;
    int signals[4];
  };


#ifdef __cplusplus
extern "C"
  {
#endif

    EAPI  int emusic_init(const char *module_filename);
    EAPI  int emusic_shutdown(void);

    EAPI  Emusic_Controller *emusic_controller_new(char *name);
    EAPI  Emusic_Controller *emusic_controller_get(char *name);
    EAPI  int emusic_controller_name_set(Emusic_Controller *controller, char *name);
    EAPI  int emusic_controller_remove(Emusic_Controller *controller);
    EAPI  int emusic_controller_mode_set(Emusic_Controller *controller, Emusic_Controller_Mode mode);
    EAPI  Emusic_Controller_Mode emusic_controller_mode_get(Emusic_Controller *controller);

    EAPI  int emusic_controller_playlist_new(Emusic_Controller *controller, char *playlist_name);
    EAPI  int emusic_controller_playlist_add(Emusic_Controller *controller, char *filename, char *playlist_name);
    EAPI  int emusic_controller_playlist_remove(Emusic_Controller *controller, char *filename);
    EAPI  int emusic_controller_playlist_remove_id(Emusic_Controller *controller, int id);
    EAPI  int emusic_controller_playlist_goto(Emusic_Controller *controller, char *filename);
    EAPI  int emusic_controller_playlist_goto_id(Emusic_Controller *controller, int id);
    EAPI  int emusic_controller_playlist_destroy(Emusic_Controller *controller, char *playlist_name);
    EAPI  int emusic_controller_playlist_load(Emusic_Controller *controller, char *filename);
    EAPI  int emusic_controller_playlist_save(Emusic_Controller *controller, char *filename);
    EAPI  Emusic_Playlist *emusic_controller_playlist_get(Emusic_Controller *controller, char *playlist_name);

    EAPI  int emusic_controller_play(Emusic_Controller *controller);
    EAPI  int emusic_controller_pause(Emusic_Controller *controller);
    EAPI  int emusic_controller_stop(Emusic_Controller *controller);
    EAPI  int emusic_controller_next(Emusic_Controller *controller);
    EAPI  int emusic_controller_prev(Emusic_Controller *controller);
    EAPI  int emusic_controller_volume_set(Emusic_Controller *controller, double volume);
    EAPI  double emusic_controller_volume_get(Emusic_Controller *controller);

    EAPI  double emusic_controller_length_get(Emusic_Controller *controller);
    EAPI  double emusic_controller_position_get(Emusic_Controller *controller);
    EAPI  int emusic_controller_position_set(Emusic_Controller *controller, double position);

    EAPI  Emusic_Capabilities emusic_controller_get_capabilities(Emusic_Controller *controller);

    EAPI  Emusic_Metadata *emusic_controller_metadata_get(Emusic_Controller *controller);
    EAPI  int emusic_controller_metadata_set(Emusic_Controller *controller, Emusic_Metadata *metadata);
    EAPI  Ecore_List *emusic_controller_metadata_get_all(Emusic_Controller *controller);

    EAPI int emusic_controller_callback_add(Emusic_Controller *controller, char *signal_name, int (*signal_cb) (void *data, int type,  void *event), void *data);

#ifdef __cplusplus

  }
#endif

#endif
